use std::collections::HashMap;
use serde::Deserialize;
use toml::map::Map;
use toml::Value;
use crate::log::logger;

const DEFAULT_PATH_ENV:&str = "/sbin:/usr/sbin:/bin:/usr/bin";

#[derive(Deserialize, Clone)]
pub struct ConfigDef {
    initrd_env: Map<String, Value>,
    env: Map<String, Value>,
    misc_fs_cfg: MiscFsConfigDef,
    fstab: Vec<FsItemDef>,
}

impl ConfigDef {
    pub fn default() -> ConfigDef {
        let mut initrd_env = Map::new();
        initrd_env.insert("path".to_string(), Value::String(DEFAULT_PATH_ENV.to_string()));
        let env = Map::new();
        let misc_fs_cfg = MiscFsConfigDef{
            runfs_option: "size=20%,mode=0755".to_string(),
            new_root: "/mnt/root_ov".to_string(),
            new_init: "/init".to_string(),
            swap: None,
        };
        let fstab = Vec::new();
        ConfigDef{ initrd_env, env, misc_fs_cfg, fstab }
    }

    fn _get_env(map: &Map<String, Value>, typ: &str) -> HashMap<String, String> {
        let mut ret_map = HashMap::new();
        for (k, v) in map {
            match v {
                Value::String(val) => {
                    ret_map.insert(k.clone(), val.clone());
                }
                _ => {
                    logger().warn("env", format!("{} var '{}' is invalid, ignored.", typ, k).as_str())
                }
            }
        }
        ret_map
    }

    pub fn get_env(&self) -> HashMap<String, String> {
        ConfigDef::_get_env(&self.env, "env")
    }

    pub fn get_initrd_env(&self) -> HashMap<String, String> {
        ConfigDef::_get_env(&self.initrd_env, "initrd_env")
    }

    pub fn get_runfs_option(&self) -> String {
        self.misc_fs_cfg.runfs_option.clone()
    }

    pub fn get_fstab(&self) -> &Vec<FsItemDef> {
        &self.fstab
    }

    pub fn get_new_root(&self) -> String {
        self.misc_fs_cfg.new_root.clone()
    }

    pub fn get_new_init(&self) -> String {
        self.misc_fs_cfg.new_init.clone()
    }

    pub fn get_swap_dev(&self) -> Option<String> {
        self.misc_fs_cfg.swap.clone()
    }
}

#[derive(Deserialize, Clone)]
pub struct MiscFsConfigDef {
    new_root: String,
    new_init: String,
    runfs_option: String,
    swap: Option<String>,
}

#[derive(Deserialize, Clone)]
pub struct FsItemDef {
    pub fstype: Option<String>,
    pub src: Option<String>,
    pub target: String,
    pub option: Option<String>,
    pub must_mount: Option<bool>,
    pub pre_mkdir: Option<Vec<String>>,
    pub post_mkdir: Option<Vec<String>>,
    pub flags: Option<Vec<String>>,
}
