mod config;
mod rumia;

use std::fmt::{Debug, Display, Formatter};
use std::fs::File;
use std::io::Read;
use std::path::Path;
use lazy_static::lazy_static;
use simple_rw_global::GlobalContainer;
use serde::de::DeserializeOwned;
use crate::cfg::config::ConfigDef;
use crate::cfg::rumia::RumiaConDef;
use crate::log::logger;

const CFG_PATH:&str = "/boot/config/initrd.toml";
const CONSOLE_CFG_PATH:&str = "/rumia_con.toml";

lazy_static! {
    pub static ref CFG : GlobalContainer<ConfigManager> = GlobalContainer::new();
}

pub struct ConfigManager {
    cfg_content: ConfigDef,
    rumia_con_content: RumiaConDef,
}

impl ConfigManager {
    #[allow(dead_code)]
    pub fn empty() -> ConfigManager{
        let rcd = RumiaConDef::default();
        let bcd = ConfigDef::default();
        ConfigManager{
            cfg_content: bcd,
            rumia_con_content: rcd,
        }
    }

    pub fn preload() -> ConfigManager{
        logger().info("cfg", format!("loading rumia_con config from '{}'...", CONSOLE_CFG_PATH).as_str());
        let rc_res = ConfigManager::load_file::<RumiaConDef>(CONSOLE_CFG_PATH);
        let rc_cfg = match rc_res {
            Ok(s) => {s}
            Err(err) => {
                ConfigManager::print_cfg_fail_info(CONSOLE_CFG_PATH, err);
                RumiaConDef::default()
            }
        };
        logger().debug("cfg", "loading rumia_con config OK.");
        ConfigManager{cfg_content: ConfigDef::default(), rumia_con_content: rc_cfg}
    }

    pub fn load_cfg(&mut self){
        logger().info("cfg", format!("loading rumia initrd config from '{}'...", CFG_PATH).as_str());
        let cfg_res = ConfigManager::load_file::<ConfigDef>(CFG_PATH);
        let main_cfg = match cfg_res {
            Ok(s) => {s}
            Err(err) => {
                ConfigManager::print_cfg_fail_info(CFG_PATH, err);
                ConfigDef::default()
            }
        };
        self.cfg_content = main_cfg;
        logger().debug("cfg", "loading rumia initrd config OK.");
    }

    fn load_file<T>(fname: &str) -> Result<T, CfgError>
        where T: DeserializeOwned,
    {
        let fpath = Path::new(fname);
        let mut file = File::open(fpath)?;
        let mut toml_str = String::new();
        file.read_to_string(&mut toml_str)?;
        let etc = toml::from_str(&toml_str)?;
        Ok(etc)
    }

    fn print_cfg_fail_info(cfgname: &str, err: CfgError) {
        logger().warn("cfg", format!("loading config '{}' failed: {:?}", cfgname, err).as_str());
        logger().warn("cfg", "rollback to default config for it instead.");
    }

    pub fn get_cfg(&self) -> &ConfigDef {
        &self.cfg_content
    }

    pub fn get_rumia_con_cfg(&self) -> &RumiaConDef {
        &self.rumia_con_content
    }
}

pub fn load_phase1() {
    let cm = ConfigManager::preload();
    CFG.set(cm);
}

pub fn load_phase2() {
    let mut cm = CFG.get_mut();
    cm.load_cfg();
}


pub enum CfgError{
    IOError(std::io::Error),
    DeserializeError(toml::de::Error),
}

impl Debug for CfgError {
    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
        match self {
            CfgError::IOError(e) => {
                write!(f, "IOError: {:?}", e)
            }
            CfgError::DeserializeError(e) => {
                write!(f, "DeserializeError: {:?}", e)
            }
        }
    }
}

impl Display for CfgError {
    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
        (self as &dyn Debug).fmt(f)
    }
}

impl std::error::Error for CfgError {
}

impl From<std::io::Error> for CfgError {
    fn from(value: std::io::Error) -> Self {
        CfgError::IOError(value)
    }
}

impl From<toml::de::Error> for CfgError {
    fn from(value: toml::de::Error) -> Self {
        CfgError::DeserializeError(value)
    }
}
