use std::any::Any;
use std::fmt::{Debug, Display, Formatter};
use std::sync::atomic::{AtomicBool, Ordering};
use std::sync::{RwLockReadGuard};
use lazy_static::lazy_static;
use simple_rw_global::{GlobalContainer, GOption};
use crate::{env, repl};

const PREFIX: &str = "[   @rumia   ]";

lazy_static!{
    static ref DEBUG_MODE: AtomicBool = AtomicBool::new(false);
    static ref IN_REPL: AtomicBool = AtomicBool::new(false);
    static ref LOGGER : GlobalContainer<LogWriter> = GlobalContainer::new();
}

pub enum LogLevel {
    #[allow(dead_code)]
    Debug,
    #[allow(dead_code)]
    Info,
    #[allow(dead_code)]
    Warn,
    #[allow(dead_code)]
    Error,
    #[allow(dead_code)]
    Fatal,
}

impl Display for LogLevel {
    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
        let s = match self {
            LogLevel::Info => "I",
            LogLevel::Warn => "W",
            LogLevel::Error => "E",
            LogLevel::Fatal => "F",
            LogLevel::Debug => "D",
        };
        write!(f, "{}", s)
    }
}

pub trait LogBackend {
    fn log(&self, module: &str ,lv: LogLevel, msg: &str);
}

pub struct ConsoleLogBackend{}

impl LogBackend for ConsoleLogBackend {
    fn log(&self, module: &str, lv: LogLevel, msg: &str) {
        println!("{} <{}> ({}) {}", PREFIX, lv, module, msg)
    }
}

pub struct LogWriter
{
    backend: Box<dyn LogBackend + Send + Sync>
}

impl LogWriter
{

    #[inline]
    #[allow(dead_code)]
    pub fn info(&self, module: &str, msg: &str) {
        self.backend.log(module,LogLevel::Info, msg);
    }

    #[inline]
    #[allow(dead_code)]
    pub fn warn(&self, module: &str, msg: &str) {
        self.backend.log(module,LogLevel::Warn, msg);
    }

    #[inline]
    #[allow(dead_code)]
    pub fn error(&self, module: &str, msg: &str) {
        self.backend.log(module,LogLevel::Error, msg);
    }

    #[inline]
    #[allow(dead_code)]
    pub fn fatal(&self, module: &str, msg: &str) {
        self.backend.log(module,LogLevel::Fatal, msg);
        //panic!("module '{}' panic with message: {}", module, msg)
        if ! IN_REPL.load(Ordering::Relaxed) {
            repl::fatal_to_shell();
        }
    }

    #[inline]
    #[allow(dead_code)]
    pub fn chk_fatal<T, E>(&self, op: Result<T, E>, module: &str, msg: &str) -> Option<T>
        where T: Any, E: Debug
    {
        match op {
            Ok(v) => Some(v),
            Err(e) => {
                self.fatal(module, format!("{}: {:?}", msg, e).as_str());
                None
            }
        }
    }

    #[inline]
    #[allow(dead_code)]
    pub fn debug(&self, module: &str, msg: &str) {
        if DEBUG_MODE.load(Ordering::Relaxed) {
            self.backend.log(module, LogLevel::Debug, msg);
        }
    }

    #[allow(dead_code)]
    pub fn is_debug_mode() -> bool {
        DEBUG_MODE.load(Ordering::Relaxed)
    }
}

pub fn logger_init(backend: Box<dyn LogBackend + Send + Sync>)
{
    LOGGER.set(LogWriter{backend})
}

pub fn logger<'a>() -> RwLockReadGuard<'a, GOption<LogWriter>>
{
    if LOGGER.is_empty() {
        panic!("use logger before init")
    }
    LOGGER.get()
}

pub fn set_debug_mode(mode: bool) {
    DEBUG_MODE.store(mode, Ordering::Relaxed);
}

pub fn print_debug_mode() {
    let mode = DEBUG_MODE.load(Ordering::Relaxed);
    let mn = if mode { "on" } else { "off" };
    logger().info("log", format!("debug mode: {}", mn).as_str());
}

pub fn set_in_repl(mode: bool) {
    IN_REPL.store(mode, Ordering::Relaxed);
}

pub fn set_debug_mode_from_env() {
    let dbg_mode = env::get_debug();
    set_debug_mode(dbg_mode);
}